//----------------------------------------------------------------------------------
// File:   MultiTextureTerrain.h
// Author: Bryan Dudash
// Email:  sdkfeedback@nvidia.com
// 
// Copyright (c) 2007 NVIDIA Corporation. All rights reserved.
//
// TO  THE MAXIMUM  EXTENT PERMITTED  BY APPLICABLE  LAW, THIS SOFTWARE  IS PROVIDED
// *AS IS*  AND NVIDIA AND  ITS SUPPLIERS DISCLAIM  ALL WARRANTIES,  EITHER  EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED  TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL  NVIDIA OR ITS SUPPLIERS
// BE  LIABLE  FOR  ANY  SPECIAL,  INCIDENTAL,  INDIRECT,  OR  CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION,  DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE  USE OF OR INABILITY  TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
//
//----------------------------------------------------------------------------------

#pragma once

#include <vector>

//--------------------------------------------------------------------------------------
// Just wraps up the DX10 objects and code necessary to render a mesh with multiple blended textures/normal maps.
//  Makes use of the Texture array to allow the cusomt tex coord to load from different maps.  the idea here is that you can bind more
//    textures than you could to the texture stages, and have each vertex sample "randomly" from any handful of the bound textures.
//--------------------------------------------------------------------------------------
class MultiTextureTerrain
{
public:

    MultiTextureTerrain();
    ~MultiTextureTerrain();

    struct Vertex
    {
        D3DXVECTOR3 pos;
        D3DXVECTOR3 norm;
        D3DXVECTOR3 tangent;
        D3DXVECTOR3 binorm;
        D3DXVECTOR4 tex0;
        D3DXVECTOR4 tex1;
    };

    void initialize(ID3D10Device *pd3dDevice);
    void draw(ID3D10Device *pd3dDevice,D3DXMATRIX &mView,D3DXMATRIX &mProj);
    void release();

    float getRadius();

    // Util function snarfed from MS samples
    HRESULT loadTextureArray( ID3D10Device* pd3dDevice, LPCTSTR* szTextureNames, int iNumTextures, ID3D10Texture2D** ppTex2D, ID3D10ShaderResourceView** ppSRV);

    // D3D10 Resources (Release() required)
    ID3D10Effect*                m_pEffect;
    ID3D10InputLayout*            m_pInputLayout;
    ID3D10Buffer *                m_pVertexBuffer;
    ID3D10Buffer *                m_pIndexBuffer;
    ID3D10Texture2D *            m_pTerrainTexture;
    ID3D10ShaderResourceView *    m_pTerrainTextureSRV;
    ID3D10Texture2D *           m_pTerrainChecker;
    ID3D10ShaderResourceView *    m_pTerrainCheckerSRV;
    ID3D10Texture2D *            m_pTerrainNormal;
    ID3D10ShaderResourceView *    m_pTerrainNormalSRV;

    // D3D10 vars (no Release() required)
    ID3D10EffectTechnique*        m_pTechnique;
    ID3D10EffectTechnique*        m_pZFirstTechnique;
    ID3D10EffectMatrixVariable* m_pmWorldViewProjection;
    ID3D10EffectMatrixVariable* m_pmWorld;
    ID3D10EffectMatrixVariable* m_pmWorldInverseTranspose;
    ID3D10EffectShaderResourceVariable* m_pTextureArrayEffectVar;
    ID3D10EffectShaderResourceVariable* m_pNormalArrayEffectVar;
    ID3D10EffectScalarVariable* m_pRenderUVsVar;
    ID3D10EffectScalarVariable* m_pUseNormalMapsVar;


    // Other
    bool m_bUseChecker;
    bool m_bRenderUVs;
    bool m_bUseNormalMaps;
    UINT m_iNumIndices;
    UINT m_iNumVertices;    
    float m_fRadius;
};